@@ -131,7 +131,7 @@ namespace win32
 		transfer >>= 10;
 		auto mbytes = transfer & 0x03ff;
 		transfer >>= 10;
-		auto gbytes = transfer & 0x03ff;
+		auto gbytes = transfer;
 
 		if (gbytes)
 			s << gbytes << " GB, ";
