@@ -1,3 +1,11 @@
+/*
+* Copyright (c) 2013-2020, The PurpleI2P Project
+*
+* This file is part of Purple i2pd project and licensed under BSD3
+*
+* See full license text in LICENSE file at top of project tree
+*/
+
 #include <stdlib.h>
 #include <string.h>
 
@@ -7,7 +15,8 @@ namespace i2p
 {
 namespace data
 {
-	static const char T32[32] = {
+	static const char T32[32] =
+	{
 		'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h',
 		'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p',
 		'q', 'r', 's', 't', 'u', 'v', 'w', 'x',
@@ -29,15 +38,16 @@ namespace data
 	* Direct Substitution Table
 	*/
 
-	static const char T64[64] = {
-		       'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H',
-		       'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P',
-		       'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X',
-		       'Y', 'Z', 'a', 'b', 'c', 'd', 'e', 'f',
-		       'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n',
-		       'o', 'p', 'q', 'r', 's', 't', 'u', 'v',
-		       'w', 'x', 'y', 'z', '0', '1', '2', '3',
-		       '4', '5', '6', '7', '8', '9', '-', '~'
+	static const char T64[64] =
+	{
+		'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H',
+		'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P',
+		'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X',
+		'Y', 'Z', 'a', 'b', 'c', 'd', 'e', 'f',
+		'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n',
+		'o', 'p', 'q', 'r', 's', 't', 'u', 'v',
+		'w', 'x', 'y', 'z', '0', '1', '2', '3',
+		'4', '5', '6', '7', '8', '9', '-', '~'
 	};
 
 	const char * GetBase64SubstitutionTable ()
@@ -67,14 +77,12 @@ namespace data
 	*
 	*/
 
-	size_t                                /* Number of bytes in the encoded buffer */
-	ByteStreamToBase64 (
-		    const uint8_t * InBuffer,           /* Input buffer, binary data */
-		    size_t    InCount,              /* Number of bytes in the input buffer */
-		    char  * OutBuffer,          /* output buffer */
-		size_t len			   /* length of output buffer */
+	size_t ByteStreamToBase64 (   /* Number of bytes in the encoded buffer */
+		const uint8_t * InBuffer, /* Input buffer, binary data */
+		size_t InCount,           /* Number of bytes in the input buffer */
+		char * OutBuffer,         /* output buffer */
+		size_t len                /* length of output buffer */
 	)
-
 	{
 		unsigned char * ps;
 		unsigned char * pd;
@@ -83,55 +91,60 @@ namespace data
 		int             i;
 		int             n;
 		int             m;
-		size_t outCount;
+		size_t          outCount;
 
 		ps = (unsigned char *)InBuffer;
-		n = InCount/3;
-		m = InCount%3;
+		n = InCount / 3;
+		m = InCount % 3;
 		if (!m)
-		     outCount = 4*n;
+			outCount = 4 * n;
 		else
-		     outCount = 4*(n+1);
+			outCount = 4 * (n + 1);
+
 		if (outCount > len) return 0;
+
 		pd = (unsigned char *)OutBuffer;
-		for ( i = 0; i<n; i++ ){
-		     acc_1 = *ps++;
-		     acc_2 = (acc_1<<4)&0x30;
-		     acc_1 >>= 2;              /* base64 digit #1 */
-		     *pd++ = T64[acc_1];
-		     acc_1 = *ps++;
-		     acc_2 |= acc_1 >> 4;      /* base64 digit #2 */
-		     *pd++ = T64[acc_2];
-		     acc_1 &= 0x0f;
-		     acc_1 <<=2;
-		     acc_2 = *ps++;
-		     acc_1 |= acc_2>>6;        /* base64 digit #3 */
-		     *pd++ = T64[acc_1];
-		     acc_2 &= 0x3f;            /* base64 digit #4 */
-		     *pd++ = T64[acc_2];
+		for ( i = 0; i < n; i++ )
+		{
+			acc_1 = *ps++;
+			acc_2 = (acc_1 << 4) & 0x30;
+			acc_1 >>= 2;                 /* base64 digit #1 */
+			*pd++ = T64[acc_1];
+			acc_1 = *ps++;
+			acc_2 |= acc_1 >> 4;         /* base64 digit #2 */
+			*pd++ = T64[acc_2];
+			acc_1 &= 0x0f;
+			acc_1 <<= 2;
+			acc_2 = *ps++;
+			acc_1 |= acc_2 >> 6;         /* base64 digit #3 */
+			*pd++ = T64[acc_1];
+			acc_2 &= 0x3f;               /* base64 digit #4 */
+			*pd++ = T64[acc_2];
 		}
-		if ( m == 1 ){
-		     acc_1 = *ps++;
-		     acc_2 = (acc_1<<4)&0x3f;  /* base64 digit #2 */
-		     acc_1 >>= 2;              /* base64 digit #1 */
-		     *pd++ = T64[acc_1];
-		     *pd++ = T64[acc_2];
-		     *pd++ = P64;
-		     *pd++ = P64;
+		if ( m == 1 )
+		{
+			acc_1 = *ps++;
+			acc_2 = (acc_1 << 4) & 0x3f; /* base64 digit #2 */
+			acc_1 >>= 2;                 /* base64 digit #1 */
+			*pd++ = T64[acc_1];
+			*pd++ = T64[acc_2];
+			*pd++ = P64;
+			*pd++ = P64;
 
 		}
-		else if ( m == 2 ){
-		     acc_1 = *ps++;
-		     acc_2 = (acc_1<<4)&0x3f;
-		     acc_1 >>= 2;              /* base64 digit #1 */
-		     *pd++ = T64[acc_1];
-		     acc_1 = *ps++;
-		     acc_2 |= acc_1 >> 4;      /* base64 digit #2 */
-		     *pd++ = T64[acc_2];
-		     acc_1 &= 0x0f;
-		     acc_1 <<=2;               /* base64 digit #3 */
-		     *pd++ = T64[acc_1];
-		     *pd++ = P64;
+		else if ( m == 2 )
+		{
+			acc_1 = *ps++;
+			acc_2 = (acc_1 << 4) & 0x3f;
+			acc_1 >>= 2;                 /* base64 digit #1 */
+			*pd++ = T64[acc_1];
+			acc_1 = *ps++;
+			acc_2 |= acc_1 >> 4;         /* base64 digit #2 */
+			*pd++ = T64[acc_2];
+			acc_1 &= 0x0f;
+			acc_1 <<= 2;                 /* base64 digit #3 */
+			*pd++ = T64[acc_1];
+			*pd++ = P64;
 		}
 
 		return outCount;
@@ -147,12 +160,11 @@ namespace data
 	*
 	*/
 
-	size_t                              /* Number of output bytes */
-	Base64ToByteStream (
-		      const char * InBuffer,           /* BASE64 encoded buffer */
-		      size_t    InCount,          /* Number of input bytes */
-		      uint8_t  * OutBuffer,	/* output buffer length */
-		  size_t len         	/* length of output buffer */
+	size_t Base64ToByteStream ( /* Number of output bytes */
+		const char * InBuffer,  /* BASE64 encoded buffer */
+		size_t InCount,         /* Number of input bytes */
+		uint8_t * OutBuffer,    /* output buffer length */
+		size_t len              /* length of output buffer */
 	)
 	{
 		unsigned char * ps;
@@ -162,42 +174,52 @@ namespace data
 		int             i;
 		int             n;
 		int             m;
-		size_t outCount;
+		size_t          outCount;
+
+		if (isFirstTime)
+			iT64Build();
+
+		n = InCount / 4;
+		m = InCount % 4;
 
-		if (isFirstTime) iT64Build();
-		n = InCount/4;
-		m = InCount%4;
 		if (InCount && !m)
-		     outCount = 3*n;
-		else {
-		     outCount = 0;
-		     return 0;
+			outCount = 3 * n;
+		else
+		{
+			outCount = 0;
+			return 0;
 		}
 
 		ps = (unsigned char *)(InBuffer + InCount - 1);
-		while ( *ps-- == P64 ) outCount--;
+		while ( *ps-- == P64 )
+			outCount--;
 		ps = (unsigned char *)InBuffer;
 
-		if (outCount > len) return -1;
+		if (outCount > len)
+			return -1;
+
 		pd = OutBuffer;
 		auto endOfOutBuffer = OutBuffer + outCount;
-		for ( i = 0; i < n; i++ ){
-		     acc_1 = iT64[*ps++];
-		     acc_2 = iT64[*ps++];
-		     acc_1 <<= 2;
-		     acc_1 |= acc_2>>4;
-		     *pd++  = acc_1;
-			 if (pd >= endOfOutBuffer) break;
+		for ( i = 0; i < n; i++ )
+		{
+			acc_1 = iT64[*ps++];
+			acc_2 = iT64[*ps++];
+			acc_1 <<= 2;
+			acc_1 |= acc_2 >> 4;
+			*pd++ = acc_1;
+			if (pd >= endOfOutBuffer)
+				break;
 
-		     acc_2 <<= 4;
-		     acc_1 = iT64[*ps++];
-		     acc_2 |= acc_1 >> 2;
-		     *pd++ = acc_2;
-			  if (pd >= endOfOutBuffer) break;
+			acc_2 <<= 4;
+			acc_1 = iT64[*ps++];
+			acc_2 |= acc_1 >> 2;
+			*pd++ = acc_2;
+			if (pd >= endOfOutBuffer)
+				break;
 
-		     acc_2 = iT64[*ps++];
-		     acc_2 |= acc_1 << 6;
-		     *pd++ = acc_2;
+			acc_2 = iT64[*ps++];
+			acc_2 |= acc_1 << 6;
+			*pd++ = acc_2;
 		}
 
 		return outCount;
@@ -206,20 +228,25 @@ namespace data
 	size_t Base64EncodingBufferSize (const size_t input_size)
 	{
 		auto d = div (input_size, 3);
-		if (d.rem) d.quot++;
-		return 4*d.quot;
+		if (d.rem)
+			d.quot++;
+
+		return 4 * d.quot;
 	}
 
 	std::string ToBase64Standard (const std::string& in)
 	{
-		auto len = Base64EncodingBufferSize (in.length ());	
-		char * str = new char[len+1];
+		auto len = Base64EncodingBufferSize (in.length ());
+		char * str = new char[len + 1];
 		auto l = ByteStreamToBase64 ((const uint8_t *)in.c_str (), in.length (), str, len);
 		str[l] = 0;
 		// replace '-' by '+' and '~' by '/'
 		for (size_t i = 0; i < l; i++)
-			if (str[i] == '-') str[i] = '+';
-			else if (str[i] == '~') str[i] = '/';
+			if (str[i] == '-')
+				str[i] = '+';
+			else if (str[i] == '~')
+				str[i] = '/';
+
 		std::string s(str);
 		delete[] str;
 		return s;
@@ -236,10 +263,10 @@ namespace data
 
 	static void iT64Build()
 	{
-		int  i;
+		int i;
 		isFirstTime = 0;
-		for ( i=0; i<256; i++ ) iT64[i] = -1;
-		for ( i=0; i<64; i++ ) iT64[(int)T64[i]] = i;
+		for ( i = 0; i < 256; i++ ) iT64[i] = -1;
+		for ( i = 0; i < 64; i++ ) iT64[(int)T64[i]] = i;
 		iT64[(int)P64] = 0;
 	}
 
@@ -302,4 +329,3 @@ namespace data
 	}
 }
 }
-
