@@ -13,6 +13,7 @@
 #include "FS.h"
 #include "Log.h"
 #include "Family.h"
+#include "Config.h"
 
 namespace i2p
 {
@@ -98,7 +99,8 @@ namespace data
 
 	void Families::LoadCertificates ()
 	{
-		std::string certDir = i2p::fs::DataDirPath("certificates", "family");
+		std::string certDir = i2p::fs::GetCertsDir() + i2p::fs::dirSep + "family";
+
 		std::vector<std::string> files;
 		int numCertificates = 0;
 
