@@ -45,10 +45,8 @@ namespace i2p
 		UpdateRouterInfo ();
 		if (IsECIES ())
 		{
-			auto initState = new i2p::crypto::NoiseSymmetricState ();
-			i2p::crypto::InitNoiseNState (*initState, GetIdentity ()->GetEncryptionPublicKey ());
-			m_InitialNoiseState.reset (initState);
-			m_ECIESSession = std::make_shared<i2p::garlic::RouterIncomingRatchetSession>(*initState);
+			i2p::crypto::InitNoiseNState (m_InitialNoiseState, GetIdentity ()->GetEncryptionPublicKey ());
+			m_ECIESSession = std::make_shared<i2p::garlic::RouterIncomingRatchetSession>(m_InitialNoiseState);
 		}
 	}
 
@@ -472,7 +470,8 @@ namespace i2p
 			uint8_t caps = m_RouterInfo.GetCaps ();
 			caps &= ~i2p::data::RouterInfo::eReachable;
 			caps |= i2p::data::RouterInfo::eUnreachable;
-			caps &= ~i2p::data::RouterInfo::eFloodfill;	// can't be floodfill
+			if (v6 || !SupportsV6 ())
+				caps &= ~i2p::data::RouterInfo::eFloodfill;	// can't be floodfill
 			m_RouterInfo.SetCaps (caps);
 		}
 		uint16_t port = 0;
@@ -486,11 +485,12 @@ namespace i2p
 				addr->ssu->introducers.clear ();
 				port = addr->port;
 			}
-		// unpiblish NTCP2 addreeses
+		// unpublish NTCP2 addreeses
 		bool ntcp2; i2p::config::GetOption("ntcp2.enabled", ntcp2);
 		if (ntcp2)
 			PublishNTCP2Address (port, false, v4, v6, false);
 		// update
+		m_RouterInfo.UpdateSupportedTransports ();
 		UpdateRouterInfo ();
 	}
 
@@ -530,6 +530,7 @@ namespace i2p
 			}
 		}
 		// update
+		m_RouterInfo.UpdateSupportedTransports ();
 		UpdateRouterInfo ();
 	}
 
@@ -568,16 +569,21 @@ namespace i2p
 			{
 				bool ntcp2; i2p::config::GetOption("ntcp2.enabled", ntcp2);
 				bool ntcp2Published; i2p::config::GetOption("ntcp2.published", ntcp2Published);
-				if (ntcp2 && ntcp2Published)
+				if (ntcp2) 
 				{
-					std::string ntcp2Host;
-					if (!i2p::config::IsDefault ("ntcp2.addressv6"))
-						i2p::config::GetOption ("ntcp2.addressv6", ntcp2Host);
+					if (ntcp2Published)
+					{	
+						std::string ntcp2Host;
+						if (!i2p::config::IsDefault ("ntcp2.addressv6"))
+							i2p::config::GetOption ("ntcp2.addressv6", ntcp2Host);
+						else
+							ntcp2Host = "::1";
+						uint16_t ntcp2Port; i2p::config::GetOption ("ntcp2.port", ntcp2Port);
+						if (!ntcp2Port) ntcp2Port = port;
+						m_RouterInfo.AddNTCP2Address (m_NTCP2Keys->staticPublicKey, m_NTCP2Keys->iv, boost::asio::ip::address::from_string (ntcp2Host), ntcp2Port);
+					}	
 					else
-						ntcp2Host = "::1";
-					uint16_t ntcp2Port; i2p::config::GetOption ("ntcp2.port", ntcp2Port);
-					if (!ntcp2Port) ntcp2Port = port;
-					m_RouterInfo.AddNTCP2Address (m_NTCP2Keys->staticPublicKey, m_NTCP2Keys->iv, boost::asio::ip::address::from_string (ntcp2Host), ntcp2Port);
+						m_RouterInfo.AddNTCP2Address (m_NTCP2Keys->staticPublicKey, m_NTCP2Keys->iv, boost::asio::ip::address(), 0, i2p::data::RouterInfo::eV6);
 				}
 			}
 			m_RouterInfo.EnableV6 ();
@@ -632,7 +638,7 @@ namespace i2p
 						m_RouterInfo.AddNTCP2Address (m_NTCP2Keys->staticPublicKey, m_NTCP2Keys->iv, boost::asio::ip::address::from_string (host), ntcp2Port);
 					}
 					else
-						m_RouterInfo.AddNTCP2Address (m_NTCP2Keys->staticPublicKey, m_NTCP2Keys->iv);
+						m_RouterInfo.AddNTCP2Address (m_NTCP2Keys->staticPublicKey, m_NTCP2Keys->iv, boost::asio::ip::address(), 0, i2p::data::RouterInfo::eV4);
 				}
 			}
 			m_RouterInfo.EnableV4 ();
@@ -679,7 +685,7 @@ namespace i2p
 			if (addr->IsPublishedNTCP2 ())
 			{
 				bool isYgg1 = i2p::util::net::IsYggdrasilAddress (addr->host);
-				if (addr->host.is_v6 () && ((isYgg && isYgg1) || (!isYgg && !isYgg1)))
+				if (addr->IsV6 () && ((isYgg && isYgg1) || (!isYgg && !isYgg1)))
 				{
 					if (addr->host != host)
 					{
@@ -736,14 +742,8 @@ namespace i2p
 			}
 		}
 		std::shared_ptr<const i2p::data::IdentityEx> oldIdentity;
-		bool rekey = m_Keys.GetPublic ()->GetSigningKeyType () == i2p::data::SIGNING_KEY_TYPE_DSA_SHA1;
-		if (!rekey && m_Keys.GetPublic ()->GetCryptoKeyType () == i2p::data::CRYPTO_KEY_TYPE_ELGAMAL)
-		{
-			// rekey routers with bandwidth = L (or default) this time
-			bool isFloodfill; i2p::config::GetOption("floodfill", isFloodfill);
-			if (!isFloodfill) rekey = true;
-		}
-		if (rekey)
+		if (m_Keys.GetPublic ()->GetSigningKeyType () == i2p::data::SIGNING_KEY_TYPE_DSA_SHA1 ||
+		    m_Keys.GetPublic ()->GetCryptoKeyType () == i2p::data::CRYPTO_KEY_TYPE_ELGAMAL)
 		{
 			// update keys
 			LogPrint (eLogInfo, "Router: router keys are obsolete. Creating new");
@@ -821,9 +821,9 @@ namespace i2p
 		i2p::HandleI2NPMessage (CreateI2NPMessage (buf, GetI2NPMessageLength (buf, len)));
 	}
 
-	bool RouterContext::HandleCloveI2NPMessage (I2NPMessageType typeID, const uint8_t * payload, size_t len)
+	bool RouterContext::HandleCloveI2NPMessage (I2NPMessageType typeID, const uint8_t * payload, size_t len, uint32_t msgID)
 	{
-		auto msg = CreateI2NPMessage (typeID, payload, len);
+		auto msg = CreateI2NPMessage (typeID, payload, len, msgID);
 		if (!msg) return false;
 		i2p::HandleI2NPMessage (msg);
 		return true;
@@ -843,10 +843,14 @@ namespace i2p
 				return;
 			}
 			buf += 4;
-			if (m_ECIESSession)
-				m_ECIESSession->HandleNextMessage (buf, len);
-			else
-				LogPrint (eLogError, "Router: Session is not set for ECIES router");
+			if (!HandleECIESx25519TagMessage (buf, len)) // try tag first
+			{	
+				// then Noise_N one-time decryption
+				if (m_ECIESSession)
+					m_ECIESSession->HandleNextMessage (buf, len);
+				else
+					LogPrint (eLogError, "Router: Session is not set for ECIES router");
+			}	
 		}
 		else
 			i2p::garlic::GarlicDestination::ProcessGarlicMessage (msg);
@@ -881,34 +885,11 @@ namespace i2p
 
 	bool RouterContext::DecryptTunnelBuildRecord (const uint8_t * encrypted, uint8_t * data)
 	{
-		if (!m_TunnelDecryptor) return false;
 		if (IsECIES ())
-		{
-			if (!m_InitialNoiseState) return false;
-			// m_InitialNoiseState is h = SHA256(h || hepk)
-			m_CurrentNoiseState.reset (new i2p::crypto::NoiseSymmetricState (*m_InitialNoiseState));
-			m_CurrentNoiseState->MixHash (encrypted, 32); // h = SHA256(h || sepk)
-			uint8_t sharedSecret[32];
-			if (!m_TunnelDecryptor->Decrypt (encrypted, sharedSecret, nullptr, false))
-			{
-				LogPrint (eLogWarning, "Router: Incorrect ephemeral public key");
-				return false;
-			}
-			m_CurrentNoiseState->MixKey (sharedSecret);
-			encrypted += 32;
-			uint8_t nonce[12];
-			memset (nonce, 0, 12);
-			if (!i2p::crypto::AEADChaCha20Poly1305 (encrypted, ECIES_BUILD_REQUEST_RECORD_CLEAR_TEXT_SIZE,
-				m_CurrentNoiseState->m_H, 32, m_CurrentNoiseState->m_CK + 32, nonce, data, ECIES_BUILD_REQUEST_RECORD_CLEAR_TEXT_SIZE, false)) // decrypt
-			{
-				LogPrint (eLogWarning, "Router: Tunnel record AEAD decryption failed");
-				return false;
-			}
-			m_CurrentNoiseState->MixHash (encrypted, ECIES_BUILD_REQUEST_RECORD_CLEAR_TEXT_SIZE + 16); // h = SHA256(h || ciphertext)
-			return true;
-		}
+			return DecryptECIESTunnelBuildRecord (encrypted, data, ECIES_BUILD_REQUEST_RECORD_CLEAR_TEXT_SIZE);
 		else
 		{
+			if (!m_TunnelDecryptor) return false;	
 			BN_CTX * ctx = BN_CTX_new ();
 			bool success = m_TunnelDecryptor->Decrypt (encrypted, data, ctx, false);
 			BN_CTX_free (ctx);
@@ -916,6 +897,42 @@ namespace i2p
 		}
 	}
 
+	bool RouterContext::DecryptECIESTunnelBuildRecord (const uint8_t * encrypted, uint8_t * data, size_t clearTextSize)
+	{	
+		// m_InitialNoiseState is h = SHA256(h || hepk)
+		m_CurrentNoiseState = m_InitialNoiseState;
+		m_CurrentNoiseState.MixHash (encrypted, 32); // h = SHA256(h || sepk)
+		uint8_t sharedSecret[32];
+		if (!m_TunnelDecryptor->Decrypt (encrypted, sharedSecret, nullptr, false))
+		{
+			LogPrint (eLogWarning, "Router: Incorrect ephemeral public key");
+			return false;
+		}
+		m_CurrentNoiseState.MixKey (sharedSecret);
+		encrypted += 32;
+		uint8_t nonce[12];
+		memset (nonce, 0, 12);
+		if (!i2p::crypto::AEADChaCha20Poly1305 (encrypted, clearTextSize, m_CurrentNoiseState.m_H, 32, 
+			m_CurrentNoiseState.m_CK + 32, nonce, data, clearTextSize, false)) // decrypt
+		{
+			LogPrint (eLogWarning, "Router: Tunnel record AEAD decryption failed");
+			return false;
+		}
+		m_CurrentNoiseState.MixHash (encrypted, clearTextSize + 16); // h = SHA256(h || ciphertext)
+		return true;
+	}
+
+	bool RouterContext::DecryptTunnelShortRequestRecord (const uint8_t * encrypted, uint8_t * data)
+	{
+		if (IsECIES ())
+			return DecryptECIESTunnelBuildRecord (encrypted, data, SHORT_REQUEST_RECORD_CLEAR_TEXT_SIZE);
+		else
+		{
+			LogPrint (eLogWarning, "Router: Can't decrypt short request record on non-ECIES router");
+			return false;
+		}	 
+	}	
+		
 	i2p::crypto::X25519Keys& RouterContext::GetStaticKeys ()
 	{
 		if (!m_StaticKeys)
