@@ -36,7 +36,7 @@ namespace data
 
 	RouterInfo::RouterInfo (const std::string& fullPath):
 		m_FullPath (fullPath), m_IsUpdated (false), m_IsUnreachable (false),
-		m_SupportedTransports (0), m_Caps (0), m_Version (0)
+		m_SupportedTransports (0), m_ReachableTransports (0), m_Caps (0), m_Version (0)
 	{
 		m_Addresses = boost::make_shared<Addresses>(); // create empty list
 		m_Buffer = new uint8_t[MAX_RI_BUFFER_SIZE];
@@ -45,7 +45,7 @@ namespace data
 
 	RouterInfo::RouterInfo (const uint8_t * buf, int len):
 		m_IsUpdated (true), m_IsUnreachable (false), m_SupportedTransports (0),
-		m_Caps (0), m_Version (0)
+		m_ReachableTransports (0), m_Caps (0), m_Version (0)
 	{
 		m_Addresses = boost::make_shared<Addresses>(); // create empty list
 		if (len <= MAX_RI_BUFFER_SIZE)
@@ -84,6 +84,7 @@ namespace data
 			m_IsUpdated = true;
 			m_IsUnreachable = false;
 			m_SupportedTransports = 0;
+			m_ReachableTransports = 0;
 			m_Caps = 0;
 			// don't clean up m_Addresses, it will be replaced in ReadFromStream
 			m_Properties.clear ();
@@ -305,9 +306,10 @@ namespace data
 					if (isHost)
 					{
 						if (address->host.is_v6 ())
-							supportedTransports |= i2p::util::net::IsYggdrasilAddress (address->host) ? eNTCP2V6Mesh :  eNTCP2V6;
+							supportedTransports |= (i2p::util::net::IsYggdrasilAddress (address->host) ? eNTCP2V6Mesh :  eNTCP2V6);
 						else
 							supportedTransports |= eNTCP2V4; 
+						m_ReachableTransports |= supportedTransports;
 					}	
 					else if (!address->published)
 					{
@@ -341,16 +343,23 @@ namespace data
 						int numValid = 0;
 						for (auto& it: address->ssu->introducers)
 						{
-							if ((!it.iExp || ts <= it.iExp) && it.iPort > 0 && 
+							if (!it.iExp) it.iExp = m_Timestamp/1000 + NETDB_INTRODUCEE_EXPIRATION_TIMEOUT;
+							if (ts <= it.iExp && it.iPort > 0 && 
 							    ((it.iHost.is_v4 () && address->IsV4 ()) || (it.iHost.is_v6 () && address->IsV6 ()))) 
 								numValid++;
 							else
 								it.iPort = 0;
 						}	
-						if (!numValid) address->ssu->introducers.resize (0);
+						if (numValid)
+							m_ReachableTransports |= supportedTransports;
+						else	
+							address->ssu->introducers.resize (0);
 					}	
 					else if (isHost && address->port)
+					{	
 						address->published = true;
+						m_ReachableTransports |= supportedTransports;
+					}	
 				}	
 			}	
 			if (supportedTransports)
@@ -546,7 +555,7 @@ namespace data
 				if (address.IsNTCP2 ())
 				{	
 					WriteString ("NTCP2", s);
-					if (address.IsPublishedNTCP2 () && !address.host.is_unspecified ())
+					if (address.IsPublishedNTCP2 () && !address.host.is_unspecified () && address.port)
 						 isPublished = true;
 					else
 					{
@@ -832,21 +841,33 @@ namespace data
 		for (const auto& it: *m_Addresses) // don't insert same address twice
 			if (*it == *addr) return;
 		m_SupportedTransports |= addr->host.is_v6 () ? eSSUV6 : eSSUV4;
+		m_ReachableTransports |= addr->host.is_v6 () ? eSSUV6 : eSSUV4;
 		m_Addresses->push_back(std::move(addr));
 	}
 
-	void RouterInfo::AddNTCP2Address (const uint8_t * staticKey, const uint8_t * iv, const boost::asio::ip::address& host, int port)
+	void RouterInfo::AddNTCP2Address (const uint8_t * staticKey, const uint8_t * iv, 
+		const boost::asio::ip::address& host, int port, uint8_t caps)
 	{
 		auto addr = std::make_shared<Address>();
 		addr->host = host;
 		addr->port = port;
 		addr->transportStyle = eTransportNTCP;		
-		addr->caps = 0;
+		addr->caps = caps;
 		addr->date = 0;
 		addr->ntcp2.reset (new NTCP2Ext ());
 		if (port) addr->published = true;
 		memcpy (addr->ntcp2->staticKey, staticKey, 32);
 		memcpy (addr->ntcp2->iv, iv, 16);
+		if (addr->IsV4 ())
+		{
+			m_SupportedTransports |= eNTCP2V4;
+			if (addr->published) m_ReachableTransports |= eNTCP2V4;
+		}	
+		if (addr->IsV6 ())
+		{
+			m_SupportedTransports |= eNTCP2V6;
+			if (addr->published) m_ReachableTransports |= eNTCP2V6;
+		}	
 		m_Addresses->push_back(std::move(addr));
 	}
 
@@ -860,6 +881,7 @@ namespace data
 				for (auto& intro: addr->ssu->introducers)
 					if (intro.iTag == introducer.iTag) return false; // already presented
 				addr->ssu->introducers.push_back (introducer);
+				m_ReachableTransports |= (addr->IsV4 () ? eSSUV4 : eSSUV6);
 				return true;
 			}
 		}
@@ -877,6 +899,8 @@ namespace data
 					if (boost::asio::ip::udp::endpoint (it->iHost, it->iPort) == e)
 					{
 						addr->ssu->introducers.erase (it);
+						if (addr->ssu->introducers.empty ())
+							m_ReachableTransports &= ~(addr->IsV4 () ? eSSUV4 : eSSUV6);
 						return true;
 					}
 			}
@@ -960,10 +984,10 @@ namespace data
 	{
 		if (!IsV6 ())
 		{	
-			m_SupportedTransports |= eSSUV6 | eNTCP2V6;
 			uint8_t addressCaps = AddressCaps::eV6;
 			if (IsV4 ()) addressCaps |= AddressCaps::eV4;
 			SetUnreachableAddressesTransportCaps (addressCaps);
+			UpdateSupportedTransports ();
 		}	
 	}
 
@@ -971,10 +995,10 @@ namespace data
 	{
 		if (!IsV4 ())
 		{	
-			m_SupportedTransports |= eSSUV4 | eNTCP2V4;
 			uint8_t addressCaps = AddressCaps::eV4;
 			if (IsV6 ()) addressCaps |= AddressCaps::eV6;
 			SetUnreachableAddressesTransportCaps (addressCaps);
+			UpdateSupportedTransports ();
 		}	
 	}
 
@@ -983,16 +1007,23 @@ namespace data
 	{
 		if (IsV6 ())
 		{
-			m_SupportedTransports &= ~(eSSUV6 | eNTCP2V6);
 			for (auto it = m_Addresses->begin (); it != m_Addresses->end ();)
 			{
 				auto addr = *it;
-				addr->caps &= ~AddressCaps::eV6;
-				if (addr->host.is_v6 ())
-					it = m_Addresses->erase (it);
+				if (addr->IsV6 ())
+				{	
+					if (addr->IsV4 ())
+					{	
+						addr->caps &= ~AddressCaps::eV6;
+						++it;
+					}	
+					else
+						it = m_Addresses->erase (it);
+				}	
 				else
 					++it;
 			}
+			UpdateSupportedTransports ();
 		}
 	}
 
@@ -1000,23 +1031,33 @@ namespace data
 	{
 		if (IsV4 ())
 		{
-			m_SupportedTransports &= ~(eSSUV4 | eNTCP2V4);
 			for (auto it = m_Addresses->begin (); it != m_Addresses->end ();)
 			{
 				auto addr = *it;
-				addr->caps &= ~AddressCaps::eV4;
-				if (addr->host.is_v4 ())
-					it = m_Addresses->erase (it);
+				if (addr->IsV4 ())
+				{	
+					if (addr->IsV6 ())
+					{	
+						addr->caps &= ~AddressCaps::eV4;
+						++it;
+					}	
+					else
+						it = m_Addresses->erase (it);
+				}	
 				else
 					++it;
 			}
+			UpdateSupportedTransports ();
 		}
 	}
 
 	void RouterInfo::EnableMesh ()
 	{
 		if (!IsMesh ())
+		{	
 			m_SupportedTransports |= eNTCP2V6Mesh;
+			m_ReachableTransports |= eNTCP2V6Mesh;
+		}	
 	}
 		
 	void RouterInfo::DisableMesh ()
@@ -1024,6 +1065,7 @@ namespace data
 		if (IsMesh ())
 		{
 			m_SupportedTransports &= ~eNTCP2V6Mesh;
+			m_ReachableTransports &= ~eNTCP2V6Mesh;
 			for (auto it = m_Addresses->begin (); it != m_Addresses->end ();)
 			{
 				auto addr = *it;
@@ -1122,9 +1164,8 @@ namespace data
 
 	bool RouterInfo::IsEligibleFloodfill () const 
 	{
-		// floodfill must be reachable somehow, >= 0.9.28 and not DSA
-		return (IsReachable () || (m_SupportedTransports & eSSUV4)) && 
-			m_Version >= NETDB_MIN_FLOODFILL_VERSION &&
+		// floodfill must be reachable by ipv4, >= 0.9.38 and not DSA
+		return IsReachableBy (eNTCP2V4 | eSSUV4) && m_Version >= NETDB_MIN_FLOODFILL_VERSION &&
 			GetIdentity ()->GetSigningKeyType () != SIGNING_KEY_TYPE_DSA_SHA1; 
 	}	
 
@@ -1150,39 +1191,43 @@ namespace data
 			});
 	}	
 
-	bool RouterInfo::IsReachableFrom (const RouterInfo& other) const
-	{
-		auto commonTransports = m_SupportedTransports & other.m_SupportedTransports;
-		if (!commonTransports) return false;
-		if (commonTransports & eNTCP2V6Mesh) return true;
-		return (bool)GetAddress (
-			[commonTransports](std::shared_ptr<const RouterInfo::Address> address)->bool
-			{	
-				if (address->IsPublishedNTCP2 ())
-				{
-					if ((commonTransports & eNTCP2V4) && address->IsV4 ()) return true;
-					if ((commonTransports & eNTCP2V6) && address->IsV6 ()) return true;
-				}	
-				else if (address->IsReachableSSU ())
-				{
-					if ((commonTransports & eSSUV4) && address->IsV4 ()) return true;
-					if ((commonTransports & eSSUV6) && address->IsV6 ()) return true;
-				}	
-				return false;
-			});
-	}	
-
 	void RouterInfo::SetUnreachableAddressesTransportCaps (uint8_t transports)
 	{
 		for (auto& addr: *m_Addresses)
 		{
 			// TODO: implement SSU
-			if (addr->transportStyle == eTransportNTCP && (!addr->IsPublishedNTCP2 () || addr->port))
+			if (addr->transportStyle == eTransportNTCP && !addr->IsPublishedNTCP2 ())
 			{
 				addr->caps &= ~(eV4 | eV6);
 				addr->caps |= transports;
 			}
 		}
 	}
+
+	void RouterInfo::UpdateSupportedTransports ()
+	{
+		m_SupportedTransports = 0;
+		m_ReachableTransports = 0;
+		for (const auto& addr: *m_Addresses)
+		{
+			uint8_t transports = 0;
+			if (addr->transportStyle == eTransportNTCP)
+			{
+				if (addr->IsV4 ()) transports |= eNTCP2V4;
+				if (addr->IsV6 ()) 
+					transports |= (i2p::util::net::IsYggdrasilAddress (addr->host) ? eNTCP2V6Mesh : eNTCP2V6);
+				if (addr->IsPublishedNTCP2 ())
+					m_ReachableTransports |= transports;
+			}
+			else if (addr->transportStyle == eTransportSSU)
+			{
+				if (addr->IsV4 ()) transports |= eSSUV4;
+				if (addr->IsV6 ()) transports |= eSSUV6;
+				if (addr->IsReachableSSU ())
+					m_ReachableTransports |= transports;
+			}	
+			m_SupportedTransports |= transports;
+		}
+	}	
 }
 }
