@@ -14,6 +14,10 @@
 #include "SSU.h"
 #include "util.h"
 
+#ifdef __linux__
+	#include <linux/in6.h>
+#endif
+
 #ifdef _WIN32
 #include <boost/winapi/error_codes.hpp>
 #endif
@@ -62,6 +66,18 @@ namespace transport
 			m_SocketV6.set_option (boost::asio::ip::v6_only (true));
 			m_SocketV6.set_option (boost::asio::socket_base::receive_buffer_size (SSU_SOCKET_RECEIVE_BUFFER_SIZE));
 			m_SocketV6.set_option (boost::asio::socket_base::send_buffer_size (SSU_SOCKET_SEND_BUFFER_SIZE));
+#ifdef __linux__
+			if (m_EndpointV6.address() == boost::asio::ip::address().from_string("::")) // only if not binded to address
+			{
+				// Set preference to use public IPv6 address -- tested on linux, not works on windows, and not tested on others
+#if (BOOST_VERSION >= 105500)
+				typedef boost::asio::detail::socket_option::integer<BOOST_ASIO_OS_DEF(IPPROTO_IPV6), IPV6_ADDR_PREFERENCES> ipv6PreferAddr;
+#else
+				typedef boost::asio::detail::socket_option::integer<IPPROTO_IPV6, IPV6_ADDR_PREFERENCES> ipv6PreferAddr;
+#endif
+				m_SocketV6.set_option (ipv6PreferAddr(IPV6_PREFER_SRC_PUBLIC | IPV6_PREFER_SRC_HOME | IPV6_PREFER_SRC_NONCGA));
+			}
+#endif
 			m_SocketV6.bind (m_EndpointV6);
 			LogPrint (eLogInfo, "SSU: Start listening v6 port ", m_EndpointV6.port());
 		}
