@@ -104,6 +104,7 @@ namespace stream
 
 	void Stream::Terminate (bool deleteFromDestination) // shoudl be called from StreamingDestination::Stop only
 	{
+		m_Status = eStreamStatusTerminated;
 		m_AckSendTimer.cancel ();
 		m_ReceiveTimer.cancel ();
 		m_ResendTimer.cancel ();
@@ -276,7 +277,20 @@ namespace stream
 		const uint8_t * optionData = packet->GetOptionData ();
 		size_t optionSize = packet->GetOptionSize ();
 		if (flags & PACKET_FLAG_DELAY_REQUESTED)
+		{	
+			if (!m_IsAckSendScheduled)
+			{
+				uint16_t delayRequested = bufbe16toh (optionData);
+				if (delayRequested > 0 && delayRequested < m_RTT)
+				{	
+					m_IsAckSendScheduled = true;
+					m_AckSendTimer.expires_from_now (boost::posix_time::milliseconds(delayRequested));
+					m_AckSendTimer.async_wait (std::bind (&Stream::HandleAckSendTimer,
+						shared_from_this (), std::placeholders::_1));
+				}	
+			}
 			optionData += 2;
+		}	
 
 		if (flags & PACKET_FLAG_FROM_INCLUDED)
 		{
@@ -793,7 +807,7 @@ namespace stream
 		if (m_CurrentRemoteLease && ts < m_CurrentRemoteLease->endDate + i2p::data::LEASE_ENDDATE_THRESHOLD)
 		{
 			std::vector<i2p::tunnel::TunnelMessageBlock> msgs;
-			for (auto it: packets)
+			for (const auto& it: packets)
 			{
 				auto msg = m_RoutingSession->WrapSingleMessage (m_LocalDestination.CreateDataMessage (
 					it->GetBuffer (), it->GetLength (), m_Port, !m_RoutingSession->IsRatchets ()));
@@ -844,12 +858,15 @@ namespace stream
 
 	void Stream::ScheduleResend ()
 	{
-		m_ResendTimer.cancel ();
-		// check for invalid value
-		if (m_RTO <= 0) m_RTO = INITIAL_RTO;
-		m_ResendTimer.expires_from_now (boost::posix_time::milliseconds(m_RTO));
-		m_ResendTimer.async_wait (std::bind (&Stream::HandleResendTimer,
-			shared_from_this (), std::placeholders::_1));
+		if (m_Status != eStreamStatusTerminated)
+		{	
+			m_ResendTimer.cancel ();
+			// check for invalid value
+			if (m_RTO <= 0) m_RTO = INITIAL_RTO;
+			m_ResendTimer.expires_from_now (boost::posix_time::milliseconds(m_RTO));
+			m_ResendTimer.async_wait (std::bind (&Stream::HandleResendTimer,
+				shared_from_this (), std::placeholders::_1));
+		}	
 	}
 
 	void Stream::HandleResendTimer (const boost::system::error_code& ecode)
@@ -1050,6 +1067,7 @@ namespace stream
 				it.second->Terminate (false); // we delete here
 			m_Streams.clear ();
 			m_IncomingStreams.clear ();
+			m_LastStream = nullptr;		
 		}
 	}
 
@@ -1058,9 +1076,16 @@ namespace stream
 		uint32_t sendStreamID = packet->GetSendStreamID ();
 		if (sendStreamID)
 		{
-			auto it = m_Streams.find (sendStreamID);
-			if (it != m_Streams.end ())
-				it->second->HandleNextPacket (packet);
+			if (!m_LastStream || sendStreamID != m_LastStream->GetRecvStreamID ())
+			{	
+				auto it = m_Streams.find (sendStreamID);
+				if (it != m_Streams.end ())
+					m_LastStream = it->second;
+				else
+					m_LastStream = nullptr;
+			}
+			if (m_LastStream)
+				m_LastStream->HandleNextPacket (packet);
 			else if (packet->IsEcho () && m_Owner->IsStreamingAnswerPings ())
 			{
 				// ping
@@ -1166,7 +1191,7 @@ namespace stream
 	{
 		auto s = std::make_shared<Stream> (m_Owner->GetService (), *this, remote, port);
 		std::unique_lock<std::mutex> l(m_StreamsMutex);
-		m_Streams[s->GetRecvStreamID ()] = s;
+		m_Streams.emplace (s->GetRecvStreamID (), s);
 		return s;
 	}
 
@@ -1174,8 +1199,8 @@ namespace stream
 	{
 		auto s = std::make_shared<Stream> (m_Owner->GetService (), *this);
 		std::unique_lock<std::mutex> l(m_StreamsMutex);
-		m_Streams[s->GetRecvStreamID ()] = s;
-		m_IncomingStreams[receiveStreamID] = s;
+		m_Streams.emplace (s->GetRecvStreamID (), s);
+		m_IncomingStreams.emplace (receiveStreamID, s);
 		return s;
 	}
 
@@ -1186,6 +1211,7 @@ namespace stream
 			std::unique_lock<std::mutex> l(m_StreamsMutex);
 			m_Streams.erase (stream->GetRecvStreamID ());
 			m_IncomingStreams.erase (stream->GetSendStreamID ());
+			if (m_LastStream == stream) m_LastStream = nullptr;
 		}
 	}
 
