@@ -24,6 +24,22 @@ namespace i2p
 {
 namespace tunnel
 {
+	void Path::Add (std::shared_ptr<const i2p::data::RouterInfo> r)
+	{
+		if (r)
+		{	
+			peers.push_back (r->GetRouterIdentity ());
+			if (r->GetVersion () < i2p::data::NETDB_MIN_SHORT_TUNNEL_BUILD_VERSION || 
+			    r->GetRouterIdentity ()->GetCryptoKeyType () != i2p::data::CRYPTO_KEY_TYPE_ECIES_X25519_AEAD)
+				isShort = false;
+		}	
+	}	
+	
+	void Path::Reverse ()
+	{
+		std::reverse (peers.begin (), peers.end ());
+	}	
+	
 	TunnelPool::TunnelPool (int numInboundHops, int numOutboundHops, int numInboundTunnels, int numOutboundTunnels):
 		m_NumInboundHops (numInboundHops), m_NumOutboundHops (numOutboundHops),
 		m_NumInboundTunnels (numInboundTunnels), m_NumOutboundTunnels (numOutboundTunnels),
@@ -142,16 +158,24 @@ namespace tunnel
 	{
 		std::vector<std::shared_ptr<InboundTunnel> > v;
 		int i = 0;
+		std::shared_ptr<InboundTunnel> slowTunnel;
 		std::unique_lock<std::mutex> l(m_InboundTunnelsMutex);
 		for (const auto& it : m_InboundTunnels)
 		{
 			if (i >= num) break;
 			if (it->IsEstablished ())
 			{
-				v.push_back (it);
-				i++;
+				if (it->IsSlow () && !slowTunnel)
+					slowTunnel = it;
+				else
+				{	
+					v.push_back (it);
+					i++;
+				}	
 			}
 		}
+		if (slowTunnel && (int)v.size () < (num/2+1))
+			v.push_back (slowTunnel);
 		return v;
 	}
 
@@ -172,13 +196,16 @@ namespace tunnel
 	{
 		if (tunnels.empty ()) return nullptr;
 		uint32_t ind = rand () % (tunnels.size ()/2 + 1), i = 0;
+		bool skipped = false;
 		typename TTunnels::value_type tunnel = nullptr;
 		for (const auto& it: tunnels)
 		{
 			if (it->IsEstablished () && it != excluded)
 			{
-				if(HasLatencyRequirement() && it->LatencyIsKnown() && !it->LatencyFitsRange(m_MinLatency, m_MaxLatency)) {
-					i ++;
+				if (it->IsSlow () || (HasLatencyRequirement() && it->LatencyIsKnown() && 
+				    !it->LatencyFitsRange(m_MinLatency, m_MaxLatency))) 
+				{
+					i++; skipped = true;
 					continue;
 				}
 				tunnel = it;
@@ -186,7 +213,8 @@ namespace tunnel
 			}
 			if (i > ind && tunnel) break;
 		}
-		if(HasLatencyRequirement() && !tunnel) {
+		if (!tunnel && skipped) 
+		{
 			ind = rand () % (tunnels.size ()/2 + 1), i = 0;
 			for (const auto& it: tunnels)
 			{
@@ -393,10 +421,14 @@ namespace tunnel
 		}
 	}
 
+	bool TunnelPool::IsExploratory () const
+	{
+		return i2p::tunnel::tunnels.GetExploratoryPool () == shared_from_this ();
+	}	
+		
 	std::shared_ptr<const i2p::data::RouterInfo> TunnelPool::SelectNextHop (std::shared_ptr<const i2p::data::RouterInfo> prevHop, bool reverse) const
 	{
-		bool isExploratory = (i2p::tunnel::tunnels.GetExploratoryPool () == shared_from_this ());
-		auto hop = isExploratory ? i2p::data::netdb.GetRandomRouter (prevHop, reverse):
+		auto hop = IsExploratory () ? i2p::data::netdb.GetRandomRouter (prevHop, reverse):
 			i2p::data::netdb.GetHighBandwidthRandomRouter (prevHop, reverse);
 
 		if (!hop || hop->GetProfile ()->IsBad ())
@@ -404,16 +436,16 @@ namespace tunnel
 		return hop;
 	}
 
-	bool StandardSelectPeers(Path & peers, int numHops, bool inbound, SelectHopFunc nextHop)
+	bool StandardSelectPeers(Path & path, int numHops, bool inbound, SelectHopFunc nextHop)
 	{
 		int start = 0;
-		auto prevHop = i2p::context.GetSharedRouterInfo ();
+		std::shared_ptr<const i2p::data::RouterInfo> prevHop = i2p::context.GetSharedRouterInfo ();
 		if(i2p::transport::transports.RoutesRestricted())
 		{
 			/** if routes are restricted prepend trusted first hop */
 			auto hop = i2p::transport::transports.GetRestrictedPeer();
 			if(!hop) return false;
-			peers.push_back(hop->GetRouterIdentity());
+			path.Add (hop);
 			prevHop = hop;
 			start++;
 		}
@@ -425,7 +457,7 @@ namespace tunnel
 				(numHops > 1 || (r->IsV4 () && (!inbound || r->IsReachable ())))) // first inbound must be reachable
 			{
 				prevHop = r;
-				peers.push_back (r->GetRouterIdentity ());
+				path.Add (r);
 				start++;
 			}
 		}
@@ -450,12 +482,12 @@ namespace tunnel
 				if (hop1) hop = hop1;
 			}	
 			prevHop = hop;
-			peers.push_back (hop->GetRouterIdentity ());
+			path.Add (hop);
 		}
 		return true;
 	}
 
-	bool TunnelPool::SelectPeers (std::vector<std::shared_ptr<const i2p::data::IdentityEx> >& peers, bool isInbound)
+	bool TunnelPool::SelectPeers (Path& path, bool isInbound)
 	{
 		int numHops = isInbound ? m_NumInboundHops : m_NumOutboundHops;
 		// peers is empty
@@ -464,27 +496,24 @@ namespace tunnel
 		{
 			std::lock_guard<std::mutex> lock(m_CustomPeerSelectorMutex);
 			if (m_CustomPeerSelector)
-				return m_CustomPeerSelector->SelectPeers(peers, numHops, isInbound);
+				return m_CustomPeerSelector->SelectPeers(path, numHops, isInbound);
 		}
 		// explicit peers in use
-		if (m_ExplicitPeers) return SelectExplicitPeers (peers, isInbound);
-		return StandardSelectPeers(peers, numHops, isInbound, std::bind(&TunnelPool::SelectNextHop, this, std::placeholders::_1, std::placeholders::_2));
+		if (m_ExplicitPeers) return SelectExplicitPeers (path, isInbound);
+		return StandardSelectPeers(path, numHops, isInbound, std::bind(&TunnelPool::SelectNextHop, this, std::placeholders::_1, std::placeholders::_2));
 	}
 
-	bool TunnelPool::SelectExplicitPeers (std::vector<std::shared_ptr<const i2p::data::IdentityEx> >& peers, bool isInbound)
+	bool TunnelPool::SelectExplicitPeers (Path& path, bool isInbound)
 	{
-		int size = m_ExplicitPeers->size ();
-		std::vector<int> peerIndicies;
-		for (int i = 0; i < size; i++) peerIndicies.push_back(i);
-		std::shuffle (peerIndicies.begin(), peerIndicies.end(), std::mt19937(std::random_device()()));
-
 		int numHops = isInbound ? m_NumInboundHops : m_NumOutboundHops;
+		if (numHops > (int)m_ExplicitPeers->size ()) numHops = m_ExplicitPeers->size ();
+		if (!numHops) return false; 
 		for (int i = 0; i < numHops; i++)
 		{
-			auto& ident = (*m_ExplicitPeers)[peerIndicies[i]];
+			auto& ident = (*m_ExplicitPeers)[i];
 			auto r = i2p::data::netdb.FindRouter (ident);
 			if (r)
-				peers.push_back (r->GetRouterIdentity ());
+				path.Add (r);
 			else
 			{
 				LogPrint (eLogInfo, "Tunnels: Can't find router for ", ident.ToBase64 ());
@@ -501,17 +530,16 @@ namespace tunnel
 		if (!outboundTunnel)
 			outboundTunnel = tunnels.GetNextOutboundTunnel ();
 		LogPrint (eLogDebug, "Tunnels: Creating destination inbound tunnel...");
-		std::vector<std::shared_ptr<const i2p::data::IdentityEx> > peers;
-		if (SelectPeers (peers, true))
+		Path path;
+		if (SelectPeers (path, true))
 		{
 			std::shared_ptr<TunnelConfig> config;
 			if (m_NumInboundHops > 0)
 			{
-				std::reverse (peers.begin (), peers.end ());
-				config = std::make_shared<TunnelConfig> (peers);
+				path.Reverse ();
+				config = std::make_shared<TunnelConfig> (path.peers, path.isShort);
 			}
-			auto tunnel = tunnels.CreateInboundTunnel (config, outboundTunnel);
-			tunnel->SetTunnelPool (shared_from_this ());
+			auto tunnel = tunnels.CreateInboundTunnel (config, shared_from_this (), outboundTunnel);
 			if (tunnel->IsEstablished ()) // zero hops
 				TunnelCreated (tunnel);
 		}
@@ -521,6 +549,11 @@ namespace tunnel
 
 	void TunnelPool::RecreateInboundTunnel (std::shared_ptr<InboundTunnel> tunnel)
 	{
+		if (IsExploratory () || tunnel->IsSlow ()) // always create new exploratory tunnel or if slow
+		{
+			CreateInboundTunnel ();
+			return;
+		}
 		auto outboundTunnel = GetNextOutboundTunnel ();
 		if (!outboundTunnel)
 			outboundTunnel = tunnels.GetNextOutboundTunnel ();
@@ -530,10 +563,9 @@ namespace tunnel
 		{
 			config = std::make_shared<TunnelConfig>(tunnel->GetPeers ());
 		}
-		if (m_NumInboundHops == 0 || config)
+		if (!m_NumInboundHops || config)
 		{
-			auto newTunnel = tunnels.CreateInboundTunnel (config, outboundTunnel);
-			newTunnel->SetTunnelPool (shared_from_this());
+			auto newTunnel = tunnels.CreateInboundTunnel (config, shared_from_this(), outboundTunnel);
 			if (newTunnel->IsEstablished ()) // zero hops
 				TunnelCreated (newTunnel);
 		}
@@ -547,15 +579,23 @@ namespace tunnel
 		if (inboundTunnel)
 		{
 			LogPrint (eLogDebug, "Tunnels: Creating destination outbound tunnel...");
-			std::vector<std::shared_ptr<const i2p::data::IdentityEx> > peers;
-			if (SelectPeers (peers, false))
+			Path path;
+			if (SelectPeers (path, false))
 			{
 				std::shared_ptr<TunnelConfig> config;
 				if (m_NumOutboundHops > 0)
-					config = std::make_shared<TunnelConfig>(peers, inboundTunnel->GetNextTunnelID (), inboundTunnel->GetNextIdentHash ());
-				auto tunnel = tunnels.CreateOutboundTunnel (config);
-				tunnel->SetTunnelPool (shared_from_this ());
-				if (tunnel->IsEstablished ()) // zero hops
+					config = std::make_shared<TunnelConfig>(path.peers, inboundTunnel->GetNextTunnelID (), inboundTunnel->GetNextIdentHash (), path.isShort);
+
+				std::shared_ptr<OutboundTunnel> tunnel;
+				if (path.isShort)
+				{
+					// TODO: implement it better
+					tunnel = tunnels.CreateOutboundTunnel (config, inboundTunnel->GetTunnelPool ());
+					tunnel->SetTunnelPool (shared_from_this ());
+				}	
+				else	
+					tunnel = tunnels.CreateOutboundTunnel (config, shared_from_this ());
+				if (tunnel && tunnel->IsEstablished ()) // zero hops
 					TunnelCreated (tunnel);
 			}
 			else
@@ -567,6 +607,11 @@ namespace tunnel
 
 	void TunnelPool::RecreateOutboundTunnel (std::shared_ptr<OutboundTunnel> tunnel)
 	{
+		if (IsExploratory () || tunnel->IsSlow ()) // always create new exploratory tunnel or if slow
+		{
+			CreateOutboundTunnel ();
+			return;
+		}
 		auto inboundTunnel = GetNextInboundTunnel ();
 		if (!inboundTunnel)
 			inboundTunnel = tunnels.GetNextInboundTunnel ();
@@ -580,8 +625,7 @@ namespace tunnel
 			}
 			if (!m_NumOutboundHops || config)
 			{
-				auto newTunnel = tunnels.CreateOutboundTunnel (config);
-				newTunnel->SetTunnelPool (shared_from_this ());
+				auto newTunnel = tunnels.CreateOutboundTunnel (config, shared_from_this ());
 				if (newTunnel->IsEstablished ()) // zero hops
 					TunnelCreated (newTunnel);
 			}
@@ -595,8 +639,7 @@ namespace tunnel
 		LogPrint (eLogDebug, "Tunnels: Creating paired inbound tunnel...");
 		auto tunnel = tunnels.CreateInboundTunnel (
 			m_NumOutboundHops > 0 ? std::make_shared<TunnelConfig>(outboundTunnel->GetInvertedPeers ()) : nullptr, 
-		    outboundTunnel);
-		tunnel->SetTunnelPool (shared_from_this ());
+		    shared_from_this (), outboundTunnel);
 		if (tunnel->IsEstablished ()) // zero hops
 			TunnelCreated (tunnel);
 	}
