@@ -1,3 +1,11 @@
+/*
+* Copyright (c) 2013-2020, The PurpleI2P Project
+*
+* This file is part of Purple i2pd project and licensed under BSD3
+*
+* See full license text in LICENSE file at top of project tree
+*/
+
 #ifndef UTIL_H
 #define UTIL_H
 
@@ -5,6 +13,7 @@
 #include <functional>
 #include <memory>
 #include <mutex>
+#include <thread>
 #include <utility>
 #include <boost/asio.hpp>
 
@@ -122,10 +131,67 @@ namespace util
 			std::mutex m_Mutex;
 	};
 
+	class RunnableService
+	{
+		protected:
+
+			RunnableService (const std::string& name): m_Name (name), m_IsRunning (false) {}
+			virtual ~RunnableService () {}
+
+			boost::asio::io_service& GetIOService () { return m_Service; }
+			bool IsRunning () const { return m_IsRunning; };
+
+			void StartIOService ();
+			void StopIOService ();
+
+		private:
+
+			void Run ();
+
+		private:
+
+			std::string m_Name;
+			volatile bool m_IsRunning;
+			std::unique_ptr<std::thread> m_Thread;
+			boost::asio::io_service m_Service;
+	};
+
+	class RunnableServiceWithWork: public RunnableService
+	{
+		protected:
+
+			RunnableServiceWithWork (const std::string& name):
+				RunnableService (name), m_Work (GetIOService ()) {}
+
+		private:
+
+			boost::asio::io_service::work m_Work;
+	};
+
+	void SetThreadName (const char *name);
+
+	template<typename T>
+	class SaveStateHelper
+	{
+		public:
+
+			SaveStateHelper (T& orig): m_Original (orig), m_Copy (orig) {};
+			~SaveStateHelper () { m_Original = m_Copy; };
+
+		private:
+
+			T& m_Original;
+			T m_Copy;
+	};
+
 	namespace net
 	{
 		int GetMTU (const boost::asio::ip::address& localAddress);
-		const boost::asio::ip::address GetInterfaceAddress(const std::string & ifname, bool ipv6=false);
+		const boost::asio::ip::address GetInterfaceAddress (const std::string & ifname, bool ipv6=false);
+		boost::asio::ip::address_v6 GetYggdrasilAddress ();
+		bool IsLocalAddress (const boost::asio::ip::address& addr);
+		bool IsInReservedRange (const boost::asio::ip::address& host);
+		bool IsYggdrasilAddress (const boost::asio::ip::address& addr);
 	}
 }
 }
